package com.yakindu.ea.examples.orderingsoftware.template.core

import java.math.BigInteger
import java.util.Date
import org.eclipse.core.resources.IFile
import org.eclipse.core.runtime.Path
import org.eclipse.uml2.uml.Class
import org.eclipse.uml2.uml.DataType
import org.eclipse.uml2.uml.Property
import org.eclipse.xtend2.lib.StringConcatenation

import static extension com.yakindu.ea.examples.orderingsoftware.template.util.JavaUtils.*

abstract class TestClassTemplate extends AbstractTemplate {
	def IFile path(Class element, IFile defaultFile) {
		if (!element.abstract) {
			return defaultFile.parent.getFile(new Path("test/PersistenceManager" + element.name.toFirstUpper + "Test." + defaultFile.fileExtension));
		}
	}
	
	def String generate(Class element) '''
		package «element.package.javaQualifiedName».test;
		
		import static org.junit.Assert.assertEquals;
		import static org.junit.Assert.assertFalse;
		import static org.junit.Assert.assertNotNull;
		import static org.junit.Assert.assertNull;
		import static org.junit.Assert.assertTrue;
		
		import java.util.UUID;
		
		import org.junit.Test;
		
		import «element.javaQualifiedName»;
		«IF element.attributes.findFirst[type instanceof Class] !== null»
			import java.util.HashMap;
			import java.util.Map;
		«ENDIF»
		«IF element.attributes.findFirst[typeof(Date).canonicalName == type.javaQualifiedName] !== null»
			import java.text.SimpleDateFormat;
		«ENDIF»
		«IF element.attributes.findFirst[typeof(BigInteger).canonicalName == type.javaQualifiedName] !== null»
			import java.math.BigInteger;
		«ENDIF»
		«FOR attribute : element.attributes.filter[type instanceof Class]»
			import «attribute.type.javaQualifiedName»;
		«ENDFOR»
		
		«printClassComment»
		public class PersistenceManager«element.name.toFirstUpper»Test extends AbstractPersistenceManagerTest {
			@Test
			public void save_should_createNew«element.name.toFirstUpper»() {
				runInPersistenceManager(manager -> {
					// given
					«printInstanceCreation(element, "newInstance1")»
					assertNull(newInstance1.getId());
					«printInstanceCreation(element, "newInstance2")»
					assertNull(newInstance2.getId());
					// when
					boolean saved1 = manager.save(newInstance1);
					boolean saved2 = manager.save(newInstance2);
					// then
					assertTrue(saved1);
					assertTrue(saved2);
					assertNotNull(newInstance1.getId());
					assertNotNull(newInstance2.getId());
					assertFalse(newInstance1.getId().equals(newInstance2.getId()));
				});
			}
			
			@Test
			public void save_should_updateExisting«element.name.toFirstUpper»() {
				// given
				«element.name» existingInstance = runInPersistenceManagerWithResult(manager -> {
					«printInstanceCreation(element, "newInstance")»
					assertTrue(manager.save(newInstance));
					return newInstance;
				});
				// when
				«IF element.attributes.findFirst[type instanceof Class] !== null»
					Map<String, UUID> idMapping = new HashMap<String, UUID>();
				«ENDIF»
				runInPersistenceManager(manager -> {
					«element.name» loadedInstance = manager.get«element.name.toFirstUpper»(existingInstance.getId());
					assertNotNull(loadedInstance);
					«printInstanceModification(element, "loadedInstance")»
					manager.save(loadedInstance);
				});
				// then
				runInPersistenceManager(manager -> {
					«element.name» loadedInstance = manager.get«element.name.toFirstUpper»(existingInstance.getId());
					assertNotNull(loadedInstance);
					«printChangedAsserts(element, "loadedInstance")»
				});
			}
			
			@Test
			public void save_should_doNothingWithNull() {
				runInPersistenceManager(manager -> {
					// given
					«element.name» newInstance = null;
					// when
					boolean saved = manager.save(newInstance);
					// then
					assertFalse(saved);
				});
			}
			
			@Test
			public void get«element.name.toFirstUpper»_should_returnExisting«element.name.toFirstUpper»() {
				// given
				«element.name» existingInstance = runInPersistenceManagerWithResult(manager -> {
					«printInstanceCreation(element, "newInstance")»
					assertTrue(manager.save(newInstance));
					return newInstance;
				});
				runInPersistenceManager(manager -> {
					// when
					«element.name» result = manager.get«element.name.toFirstUpper»(existingInstance.getId());
					// then
					assertNotNull(result.getId());
					assertEquals(existingInstance.getId(), result.getId());
				});
			}
			
			@Test
			public void get«element.name.toFirstUpper»_should_returnNullForNotExisting«element.name.toFirstUpper»() {
				runInPersistenceManager(manager -> {
					// given
					UUID id = UUID.randomUUID();
					// when
					«element.name» result = manager.get«element.name.toFirstUpper»(id);
					// then
					assertNull(result);
				});
			}
			
			@Test
			public void get«element.name.toFirstUpper»_should_returnNullForNotNullId() {
				runInPersistenceManager(manager -> {
					// given
					UUID id = null;
					// when
					«element.name» result = manager.get«element.name.toFirstUpper»(id);
					// then
					assertNull(result);
				});
			}
		}
	'''
	
	protected def String printClassComment() '''
		/**
		 * This file was generated by {@link «class.name»}!
		 */
	'''
	
	protected def String printInstanceCreation(Class element, String instanceName) '''
		«element.name» «instanceName» = new «element.name»();
		«FOR attribute : element.ownedAttributes SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
			«IF attribute.type instanceof DataType»
				«printInitialDatatypeAttributeAssignment(attribute, instanceName)»
			«ELSE»
				«IF 1 == attribute.upper»
					«printInitialSingleValuedInstanceAttributeAssignment(attribute, instanceName)»
				«ELSE»
					«printInitialMultiValuedInstanceAttributeAssignment(attribute, instanceName)»
				«ENDIF»
			«ENDIF»
		«ENDFOR»
	'''
	
	protected def String printInitialDatatypeAttributeAssignment(Property attribute, String instanceName) '''
		«IF typeof(Date).canonicalName == attribute.type.javaQualifiedName»
			«instanceName».set«attribute.name.toFirstUpper»(new SimpleDateFormat("yyyy-MM-dd").parse("2019-12-06"));
		«ELSE»
			«IF typeof(BigInteger).canonicalName == attribute.type.javaQualifiedName»
				«instanceName».set«attribute.name.toFirstUpper»(new BigInteger("42"));
			«ELSE»
				«instanceName».set«attribute.name.toFirstUpper»("Initial Value");
			«ENDIF»
		«ENDIF»
	'''
	
	protected def String printInitialSingleValuedInstanceAttributeAssignment(Property attribute, String instanceName) '''
		{
			«attribute.type.name» new«attribute.name.toFirstUpper»Value = new «attribute.type.name»();
			«instanceName».set«attribute.name.toFirstUpper»(new«attribute.name.toFirstUpper»Value);
			manager.save(new«attribute.name.toFirstUpper»Value);
		}
	'''
	
	protected def String printInitialMultiValuedInstanceAttributeAssignment(Property attribute, String instanceName) '''
		{
			«attribute.type.name» new«attribute.name.toFirstUpper»Value = new «attribute.type.name»();
			«instanceName».get«attribute.name.toFirstUpper»().add(new«attribute.name.toFirstUpper»Value);
			manager.save(new«attribute.name.toFirstUpper»Value);
		}
	'''
	
	protected def String printInstanceModification(Class element, String instanceName) '''
		«FOR attribute : element.ownedAttributes SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
			«IF attribute.type instanceof DataType»
				«printChangeDatatypeAttributeAssignment(attribute, instanceName)»
			«ELSE»
				«IF 1 == attribute.upper»
					«printChangedSingleValuedInstanceAttributeAssignment(attribute, instanceName)»
				«ELSE»
					«printChangedMultiValuedInstanceAttributeAssignment(attribute, instanceName)»
				«ENDIF»
			«ENDIF»
		«ENDFOR»
	'''
	
	protected def String printChangeDatatypeAttributeAssignment(Property attribute, String instanceName) '''
		«IF typeof(Date).canonicalName == attribute.type.javaQualifiedName»
			«instanceName».set«attribute.name.toFirstUpper»(new SimpleDateFormat("yyyy-MM-dd").parse("2019-12-24"));
		«ELSE»
			«IF typeof(BigInteger).canonicalName == attribute.type.javaQualifiedName»
				«instanceName».set«attribute.name.toFirstUpper»(new BigInteger("4711"));
			«ELSE»
				«instanceName».set«attribute.name.toFirstUpper»("Changed Value");
			«ENDIF»
		«ENDIF»
	'''
	
	protected def String printChangedSingleValuedInstanceAttributeAssignment(Property attribute, String instanceName) '''
		{
			«attribute.type.name» new«attribute.name.toFirstUpper»Value = new «attribute.type.name»();
			«instanceName».set«attribute.name.toFirstUpper»(new«attribute.name.toFirstUpper»Value);
			manager.save(new«attribute.name.toFirstUpper»Value);
			idMapping.put("«attribute.name»", new«attribute.name.toFirstUpper»Value.getId());
		}
	'''
	
	protected def String printChangedMultiValuedInstanceAttributeAssignment(Property attribute, String instanceName) '''
		{
			«attribute.type.name» new«attribute.name.toFirstUpper»Value = new «attribute.type.name»();
			«instanceName».get«attribute.name.toFirstUpper»().clear();
			«instanceName».get«attribute.name.toFirstUpper»().add(new«attribute.name.toFirstUpper»Value);
			manager.save(new«attribute.name.toFirstUpper»Value);
			idMapping.put("«attribute.name»", new«attribute.name.toFirstUpper»Value.getId());
		}
	'''
	
	protected def String printChangedAsserts(Class element, String instanceName) '''
		«FOR attribute : element.ownedAttributes SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
			«IF attribute.type instanceof DataType»
				«printChangedDatatypeAssert(attribute, instanceName)»
			«ELSE»
				«IF 1 == attribute.upper»
					«printChangedSingleValuedInstanceAssert(attribute, instanceName)»
				«ELSE»
					«printChangedMultiValuedInstanceAssert(attribute, instanceName)»
				«ENDIF»
			«ENDIF»
		«ENDFOR»
	'''
	
	protected def String printChangedDatatypeAssert(Property attribute, String instanceName) '''
		«IF typeof(Date).canonicalName == attribute.type.javaQualifiedName»
			assertEquals(new SimpleDateFormat("yyyy-MM-dd").parse("2019-12-24"), loadedInstance.get«attribute.name.toFirstUpper»());
		«ELSE»
			«IF typeof(BigInteger).canonicalName == attribute.type.javaQualifiedName»
				assertEquals(new BigInteger("4711"), loadedInstance.get«attribute.name.toFirstUpper»());
			«ELSE»
				assertEquals("Changed Value", loadedInstance.get«attribute.name.toFirstUpper»());
			«ENDIF»
		«ENDIF»
	'''
	
	protected def String printChangedSingleValuedInstanceAssert(Property attribute, String instanceName) '''
		assertEquals(idMapping.get("«attribute.name»"), loadedInstance.get«attribute.name.toFirstUpper»().getId());
	'''
	
	protected def String printChangedMultiValuedInstanceAssert(Property attribute, String instanceName) '''
		assertEquals(1, loadedInstance.get«attribute.name.toFirstUpper»().size());
		assertEquals(idMapping.get("«attribute.name»"), loadedInstance.get«attribute.name.toFirstUpper»().get(0).getId());
	'''
}